// This project was based on the ServoActuator.ino project from the mySensor samples
//
      // ServoActuator.ino
      // Example showing how to create an atuator for a servo.
      // Connect red to +5V, Black or brown to GND and the last cable to Digital pin 3.
      // The servo consumes much power and should probably have its own powersource.'
      // The arduino might spontanally restart if too much power is used (happend
      // to me when servo tried to pass the extreme positions = full load).
      // Contribution by: Derek Macias
//

// May 2014 ML
// This project will allow the automatation of a window blind (1" Faux Wood Style) tilting feature,
// via a VeraLite, using vera scenes with Day/Night detection, Light Levels or Tempature Level.
//
// This was my learning project to make use of wonderful software library of the mySensors project
// that allows users of the VERA automation system to interface with home made sensors using the
// arduino platform.

// See attached wiring schematic for building the multi-feature sensor controller.

// Included a seperate debug process that can be enabled and disabled without conflicting
// with the base mySensor DEBUG (that is located in the config.h file).  The mySensor library
// has been well debugged and we don't need to see their debug info if we share their debug routine.
//#include <ServoActuator/debug.h>

// Include the various mySensor library files
#include <Sensor.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <Servo.h> 
#include <RF24.h>

// You must download and copy this to your libraries folder
// https://github.com/pfeerick/elapsedMillis/archive/master.zip
#include <elapsedMillis.h>

// This project will make use of a Dallas tempature sensor
// Include the Temperature headers that are located in the \Arduino\libraries\DallasTemperature location
#include <DallasTemperature.h>
#include <OneWire.h>
                                                                                        // Enable/Disable project only debug logging 
//#define PDEBUG 1                                                                      // Comment this line for no debug printing when project is ready for deployment
                                                                                        
// Define the Arduino Pins to be used in this project                                                                                        
#define LIGHT_ANALOG_IN_PIN 3                                                           // Analog for light sensor
#define TEMP_DIGITAL_IN_PIN 7                                                           // Digital for Temperature sensor
#define SERVO_DIGITAL_OUT_PIN 3                                                         // Digital for Servo motor (simulate PWM)                                            
#define BUTTON_DIGITAL_IN_PIN 4                                                         // Digital for Override switch (on/off

#define SERVO_MIN 0                                                                     // Fine tune your servos min. 0-180
#define SERVO_MAX 180                                                                   // Fine tune your servos max. 0-180
#define DETACH_DELAY 900                                                                // Tune this to let your motor movement finish before detaching the servo
#define CHILD_SERVO_ID 10                                                               // Id of the sensor servo child
#define CHILD_LIGHT_ID 11                                                               // Id of the sensor light child
#define CHILD_TEMP_ID 12                                                                // Id of the sensor temperature child
#define sensor_interval 60000                                                           // the interval to check the external sensors for changes (60 seconds)
#define temperature_difference 2                                                        // number of temperature units in which the temperature must differ before recording
#define light_level_difference 5                                                        // number of light units in which the light must differ before recording

// Temperature defines
#define MAX_ATTACHED_DS18B20 16                                                         // 1 to 16 DS18B20 sensors


OneWire oneWire(TEMP_DIGITAL_IN_PIN);                                                   // initialize the temperature wire bus
DallasTemperature sensors(&oneWire);                                                    // reserve space to hold our readings

Sensor gw;                                                                              // create s sensor object
Servo myservo;                                                                          // create servo object to control a servo 

elapsedMillis timer0;                                                                   // timer to see when to interogate the external sensors SEE INCLUDE ABOVE FOR DOWNLOAD

unsigned long timeOfLastServoChange = 0;                                                // holds the time in milliseconds when the servo last changed position
                                                                                        // allows for the detachment of the servo after a small delay (see DETACH_DELAY) so
                                                                                        // power is not constantly being applied to the servo prevent battery, if used, from being drained
                                                                                        // is sketch running under metric standard
                                                                                        
int lightSensorLastReading = 0;                                                         // holds the last light sensor reading
int photocellReading = 0;                                                               // the analog reading from the sensor divider
int buttonState = 0;                                                                    // variable for reading the pushbutton status								          
int numSensors = 0;                                                                     // number of temperature sensors we located during setup
float lastTemperature[MAX_ATTACHED_DS18B20];                                            // holds the last temp sensor(s) readings
bool attachedServo = false;                                                             // tracks whether or not the servo is current attached            
bool metric = false;                                                                    // Use metric Celsius or Fahrenheit when working with temperature values
bool button_override_state = false;                                                     // tracks whether the override button is pressed	

// Setup the debugging process
// If PDEBUG is defined (not removed) in above code then stream debug data to the serial window.
#ifdef PDEBUG                                                                                     
  void StreamPrint_progmem(Print &out,PGM_P format,...)
  {                                                                                   // http://www.utopiamechanicus.com/article/low-memory-serial-print/
                                                                                      // program memory version of printf - copy of format string and result share a buffer
                                                                                      // so as to avoid too much memory use. There is no println() version – instead, 
    char formatString[128], *ptr;                                                     // add \n or \r to your strings
    strncpy_P( formatString, format, sizeof(formatString) );                          // copy in from program mem
                                                                                      // null terminate - leave last char since we might need it in worst case for result's \0
    formatString[ sizeof(formatString)-2 ]='\0'; 
    ptr=&formatString[ strlen(formatString)+1 ];                                      // our result buffer...
    va_list args;
    va_start (args,format);
    vsnprintf(ptr, sizeof(formatString)-1-strlen(formatString), formatString, args );
    va_end (args);
    formatString[ sizeof(formatString)-1 ]='\0'; 
    out.print(ptr);
  } 
  #define pdebug(format, ...) StreamPrint_progmem(Serial,PSTR(format),##__VA_ARGS__)
#else
  #define pdebug(x,...)
#endif

void setup() 
{ 

  sensors.begin();                                                                     // initialize the temperature object
  gw.begin();                                                                          // initialize the sensor object

                                                                                       pdebug("Staring Setup\n"); 
  gw.sendSketchInfo("Servo/Temp/Lux", "1.0");                                          // Send the sketch version information to the gateway and Controller
  gw.sendSensorPresentation(CHILD_SERVO_ID, S_COVER);                                  // Register all sensors to gw (will be created as child device)
  gw.sendSensorPresentation(CHILD_LIGHT_ID, S_LIGHT_LEVEL);                            // Register all sensors to gw (will be created as child device)
  numSensors = sensors.getDeviceCount();                                               // Get number of temperature sensors on the bus													
  for (int i=0; i<numSensors && i<MAX_ATTACHED_DS18B20; i++)
  { 
    gw.sendSensorPresentation(i+CHILD_TEMP_ID, S_TEMP);                                // Register temperature sensors to gw (will be created as child devices)
  }	
                                                                                       // Fetch servo state at startup
                                                                                       // I use variable1 in the vera-advance section of the sensor to determine
                                                                                       // how the blinds should be tilted when the sensor first boots up.
                                                                                       // Example:  I default to a value of 0 (CLOSED).  This value can be anything
                                                                                       // between 0 (CLOSED) to 50 (FULL OPEN) 100 (CLOSED) (My Binds have 180 degrees
                                                                                       // of tilt form closed [0] to open [90] back to closed [180] (SEE BLIND NOTES BELOW)
                                                                                       // If power is lost at night, I don't want the blinds going into a open state
                                                                                       // as the relay initializes with the return of power.
  gw.getStatus(CHILD_SERVO_ID, V_VAR1);
  setRelayStatus(gw.getMessage());                                                     // Wait here until status message arrive from gw. You are locked here until GW reply is received
                                                                                       // if debugging send data to serial monitor
                                                                                        pdebug("Variable1 read from vera object %d\n", V_VAR1);

                                                                                        
  gw.getStatus(CHILD_SERVO_ID, V_DIMMER);                                               // Fetch last dimmer postion state at startup (LoadLevelStatus in vera object)
  setRelayStatus(gw.getMessage());                                                      // Wait here until status message arrive from gw. You are locked here until GW replay is received
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("LoadLevelStatus read from vera object %d\n", V_DIMMER);
                                                                                        
  timer0 = 0;                                                                           // clear the timer at the end of startup
  pinMode(BUTTON_DIGITAL_IN_PIN, INPUT);                                                // initialize the pushbutton pin as an input:
} 

void loop() 
{ 
  if (gw.messageAvailable())
  {
    message_s message = gw.getMessage();					        // Get the new message that the Gateway (GW) as forwarded to this sensor non blocking
    if (button_override_state == false)
     {                                                                                  // Override blind close button has been pressed ignore all automation requests
        setRelayStatus(message);							// Process the message from the Gateway (GW)
     }
  }
   
  buttonState = digitalRead(BUTTON_DIGITAL_IN_PIN);                                     // Read the state of the push button value:  
                                                                                       
                                                                                        // Have we used the servo resently.  If so has enough time passed after sending
  if (attachedServo && millis() - timeOfLastServoChange > DETACH_DELAY)                 // the last position to go ahead and detact the servo, so we are consumming power
  {
    myservo.detach();                                                                   // Detach the servo
    attachedServo = false;                                                              // Set local detachment state of the servo object
  }
                                                                                        // Has our timer interval triggered if so check out sensors for changes
  if (timer0 > sensor_interval) 
  {
    checkLightSensor();                                                                 // do the light sensor check
    checkTempSensors();                                                                 // do the temperature sensor(s) check
    timer0 -= sensor_interval;                                                          // reset the timer0 
  }                                                                                     
                                                                                        // check if the switch is in the override position.                                                                                     
   if (buttonState == HIGH) {                                                           // if it is, the buttonState is HIGH:
       if (button_override_state == false) {
          myservo.attach(SERVO_DIGITAL_OUT_PIN);                                        // we have an action to perform so get the servo ready
          attachedServo = true;                                                         pdebug("OVERRIDE Switch is in closed state\n");
          myservo.write(SERVO_MIN);                                                     // move the servo to full CLOSED [0] position
          gw.sendVariable(CHILD_SERVO_ID, V_DIMMER, SERVO_MIN);                         // Let the Gateway (GW) and vera know what we changed 
          button_override_state = true;                                                 // Switch is in the closed state, closing the blinds until switch is in open state
          timeOfLastServoChange = millis();                                             // allow time for the servo to finish it movement
       }
   } 
   else { 
         if (button_override_state == true) {
                                                                                        pdebug("OVERRIDE Switch is in open state\n");
           button_override_state = false;                                               // Switch has been opened let automation take over control again
         }
   }
} 

void checkLightSensor()
{

  photocellReading = analogRead(LIGHT_ANALOG_IN_PIN);                                   // Read the light sensor  
                                                                                        // here we take a value between 0 and 1023 photocellReading  
  int val_light = map(photocellReading, 0, 1023, 0, 100);                               // and map it to a value suitable for the a percetage (0 to 100) representation
  if (((val_light) < lightSensorLastReading - light_level_difference) || ((val_light ) > lightSensorLastReading + light_level_difference))
   {
      gw.sendVariable(CHILD_LIGHT_ID, V_LIGHT_LEVEL, val_light);                        // Send light level reading to Gateway (gw)			
      lightSensorLastReading = val_light;                                               // Record last temperature value
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("Light change %d\n", val_light); 
   }   
}

void checkTempSensors()
{
  sensors.requestTemperatures();                                                        // Fetch temperatures from Dallas
  delay(100);
  for (int i=0; i<numSensors && i<MAX_ATTACHED_DS18B20; i++) 
  {
                                                                                        // Fetch and round temperature to one decimal
    float temperature = static_cast<float>(static_cast<int>((metric?sensors.getTempCByIndex(i):sensors.getTempFByIndex(i)) * 10.)) / 10.;	
    gw.powerUp();                                                                       // Powerup introduces a small delay (which is missing in radio.write powerup)
                                                                                        // if temperature differs by 2 units we forward on to Gateway (gw)
    if (temperature < (lastTemperature[i] - temperature_difference) || temperature > (lastTemperature[i] + temperature_difference))
    {
      gw.sendVariable(i+CHILD_TEMP_ID, V_TEMP, temperature, 1);                         // Send temperature reading to Gateway (gw)
      lastTemperature[i]=temperature;                                                   // Record last temperature value
      char strout0[8];                                                                  // Reserve string space
      dtostrf(temperature,3, 2, strout0);                                               // Convert float to string
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("Temperature change %s\n",strout0); 
    }
  }
}

void setRelayStatus(message_s message)
{
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("message.header.type %d\n", message.header.type);
  myservo.attach(SERVO_DIGITAL_OUT_PIN);                                                // we have an action to perform so get the servo ready
  attachedServo = true;                                                                 // Set local attachment state of the servo object
                                                                                      														
  if (message.header.type==V_DIMMER)                                                    // Determine what type of message from Gateway (GW) we need to react to	
  {                                                                                       
    int val = atoi(message.data);                                                       // We are dealing with V_DIMMER slider change of position
                                                                                        // 0   = Closed                               [0 degrees]
                                                                                        // #   = Some Percent Open between 1 and 49   [1-89 degrees]
                                                                                        // 50  = Open  (SEE BLIND NOTES BELOW)        [90 degrees]
                                                                                        // #   = Some Percent Open between 51 and 99  [91-179 degrees]
                                                                                        // 100 = Closed                               [180 degrees]
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("V_DIMMER (slider) value %d\n", val);
                                                                                        // here we take a value between 0% and 100% passed in from the Gateway (GW)
                                                                                        // and map it to a value suitable for the servo (0 to 180 degrees of rotation
    int val_map = map(val, 0, 100, 0, 180);
    myservo.write(val_map);								// move the servo to new mapped position
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("Servo (mapped) value %d\n", val_map); 

  } 
  else if (message.header.type==V_UP) 
  {
                                                                                        // We are dealing with V_UP button being clicked
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("V_UP (button) value %d\n", SERVO_MAX);
    myservo.write(SERVO_MAX);                                                           // move the servo to full OPEN (180) position
    gw.sendVariable(CHILD_SERVO_ID, V_DIMMER, SERVO_MAX);                               // Let the Gateway (GW) and vera know what we changed

  } 
  else if (message.header.type==V_DOWN) 
  {
                                                                                        // We are dealing with V_DOWN button being clicked
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("V_DOWN (button) value %d\n", SERVO_MIN);
    myservo.write(SERVO_MIN);                                                           // move the servo to full CLOSED (0) position
    gw.sendVariable(CHILD_SERVO_ID, V_DIMMER, SERVO_MIN);                               // Let the Gateway (GW) and vera know what we changed
  } 
  else if (message.header.type==V_STOP) 
  {
                                                                                        // We are dealing with V_DOWN button being clicked
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("V_STOP (button)\n"); 
    myservo.detach();                                                                   // Detach the servo
    attachedServo = false;                                                              // Set local detachment state of the servo object
  } 
  else if (message.header.type==V_VAR1) 
  {
                                                                                        // We are dealing with Change in V_VAR1 (Variable1) in the vera
    int val = atoi(message.data);                                                       // Read the new value as a percentage (0% to 100%) 
    int val_map = map(val, 0, 100, 0, 180);                                             // Map it to a value suitable for the servo (0 to 180 degrees of rotation   
    myservo.write(val_map);                                                             // move the servo to new position		
    gw.sendVariable(CHILD_SERVO_ID, V_DIMMER, val);                                     // Let the Gateway (GW) and vera know what we changed
  }

  timeOfLastServoChange = millis();                                                     // mark the last time we changed the servo position so we can detatch it after a small delay

                                                                                        // While we are at it lets check the current voltage level, in case we are on battery power
                                                                                        // this is not 100% acurate, but will give you a general idea of the battery power level
  int millVolt = readVcc();                                                             // get the voltage level in milli-volts. In this sample I am using a 5-volt poser supply
                                                                                        // checking the voltage with a volt-meter and this power supply it sends 5.15v into the arduino
  int powerPercentage = map(millVolt, 0, 5150, 0, 100);                                 // take the milli-volt reading and map it to a 0 to 100% percentage based on what we expect
                                                                                        // if debugging send data to serial monitor
                                                                                        pdebug("Current power level (milli-volts) is %d\n", millVolt); 
                                                                                        pdebug("Current power level (percentage) is %d\n", powerPercentage);  
  gw.sendBatteryLevel(powerPercentage);                                                 // send the percentage value back to the Gateway (GW) so vera can show the battery level
  gw.sendVariable(CHILD_SERVO_ID, V_VAR2, millVolt);                                    // also I like to send the milli-volt reading back to the sensor object so I can look at
                                                                                        // it using the vera interface.  it is stored in the Variable2 under the advanced tab of
                                                                                        // the sensor.

}

                                                                                        // this method was pulled from http://www.mysensors.org/git/Arduino/libraries/MySensors/examples/BatteryPoweredSensor/BatteryPoweredSensor.ino
                                                                                        // and included in this sketch 
long readVcc()
{
                                                                                        // This is an example that demonstrates how to report the battery level for a sensor
                                                                                        // Instructions for measuring battery capacity on A0 are available in the follwoing forum
                                                                                        // thread: http://forum.micasaverde.com/index.php/topic,20078.0.html
  long result;
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);                               // Read 1.1V reference against AVcc
  delay(2);                                                                             // Wait for Vref to settle
  ADCSRA |= _BV(ADSC);                                                                  // Convert
  while (bit_is_set(ADCSRA,ADSC));
  result = ADCL;
  result |= ADCH<<8;
  result = 1125300L / result;                                                           // Back-calculate AVcc in mV
  return result;
}
 
 

